@@ -1,29 +1,30 @@
-"""A class representing the type adapter."""
+"""Type adapter specification."""
 from __future__ import annotations as _annotations
 
 import sys
 from dataclasses import is_dataclass
-from typing import TYPE_CHECKING, Any, Dict, Generic, Iterable, Set, TypeVar, Union, overload
+from typing import TYPE_CHECKING, Any, Dict, Generic, Iterable, Set, TypeVar, Union, cast, final, overload
 
 from pydantic_core import CoreSchema, SchemaSerializer, SchemaValidator, Some
-from typing_extensions import Literal, is_typeddict
+from typing_extensions import Literal, get_args, is_typeddict
 
 from pydantic.errors import PydanticUserError
 from pydantic.main import BaseModel
 
-from ._internal import _config, _core_utils, _discriminated_union, _generate_schema, _typing_extra
+from ._internal import _config, _generate_schema, _typing_extra
 from .config import ConfigDict
 from .json_schema import (
     DEFAULT_REF_TEMPLATE,
-    DefsRef,
     GenerateJsonSchema,
     JsonSchemaKeyT,
     JsonSchemaMode,
     JsonSchemaValue,
 )
+from .plugin._schema_validator import create_schema_validator
 
 T = TypeVar('T')
 
+
 if TYPE_CHECKING:
     # should be `set[int] | set[str] | dict[int, IncEx] | dict[str, IncEx] | None`, but mypy can't cope
     IncEx = Union[Set[int], Set[str], Dict[int, Any], Dict[str, Any]]
@@ -31,37 +32,39 @@ if TYPE_CHECKING:
 
 def _get_schema(type_: Any, config_wrapper: _config.ConfigWrapper, parent_depth: int) -> CoreSchema:
     """`BaseModel` uses its own `__module__` to find out where it was defined
-    and then look for symbols to resolve forward references in those globals.
+    and then looks for symbols to resolve forward references in those globals.
     On the other hand this function can be called with arbitrary objects,
-    including type aliases where `__module__` (always `typing.py`) is not useful.
+    including type aliases, where `__module__` (always `typing.py`) is not useful.
     So instead we look at the globals in our parent stack frame.
 
     This works for the case where this function is called in a module that
     has the target of forward references in its scope, but
-    does not work for more complex cases.
+    does not always work for more complex cases.
 
     For example, take the following:
 
     a.py
     ```python
-    from typing import List, Dict
+    from typing import Dict, List
+
     IntList = List[int]
     OuterDict = Dict[str, 'IntList']
     ```
 
     b.py
-    ```python
-    from pydantic import TypeAdapter
+    ```python test="skip"
     from a import OuterDict
+
+    from pydantic import TypeAdapter
+
     IntList = int  # replaces the symbol the forward reference is looking for
     v = TypeAdapter(OuterDict)
-    v({"x": 1})  # should fail but doesn't
+    v({'x': 1})  # should fail but doesn't
     ```
 
-    If OuterDict were a `BaseModel`, this would work because it would resolve
+    If `OuterDict` were a `BaseModel`, this would work because it would resolve
     the forward reference within the `a.py` namespace.
-    But `TypeAdapter(OuterDict)`
-    can't know what module OuterDict came from.
+    But `TypeAdapter(OuterDict)` can't determine what module `OuterDict` came from.
 
     In other words, the assumption that _all_ forward references exist in the
     module we are being called from is not technically always true.
@@ -76,7 +79,7 @@ def _get_schema(type_: Any, config_wrapper: _config.ConfigWrapper, parent_depth:
     global_ns.update(local_ns or {})
     gen = _generate_schema.GenerateSchema(config_wrapper, types_namespace=global_ns, typevars_map={})
     schema = gen.generate_schema(type_)
-    schema = gen.collect_definitions(schema)
+    schema = gen.clean_schema(schema)
     return schema
 
 
@@ -95,46 +98,101 @@ def _getattr_no_parents(obj: Any, attribute: str) -> Any:
         raise AttributeError(attribute)
 
 
+def _type_has_config(type_: Any) -> bool:
+    """Returns whether the type has config."""
+    try:
+        return issubclass(type_, BaseModel) or is_dataclass(type_) or is_typeddict(type_)
+    except TypeError:
+        # type is not a class
+        return False
+
+
+@final
 class TypeAdapter(Generic[T]):
-    """A class representing the type adapter.
+    """Usage docs: https://docs.pydantic.dev/2.7/concepts/type_adapter/
+
+    Type adapters provide a flexible way to perform validation and serialization based on a Python type.
 
     A `TypeAdapter` instance exposes some of the functionality from `BaseModel` instance methods
     for types that do not have such methods (such as dataclasses, primitive types, and more).
 
-    Note that `TypeAdapter` is not an actual type, so you cannot use it in type annotations.
+    **Note:** `TypeAdapter` instances are not types, and cannot be used as type annotations for fields.
 
     Attributes:
         core_schema: The core schema for the type.
-        validator: The schema validator for the type.
+        validator (SchemaValidator): The schema validator for the type.
         serializer: The schema serializer for the type.
     """
 
-    if TYPE_CHECKING:
+    @overload
+    def __init__(
+        self: TypeAdapter[T],
+        type: type[T],
+        *,
+        config: ConfigDict | None = ...,
+        _parent_depth: int = ...,
+        module: str | None = ...,
+    ) -> None:
+        ...
 
-        @overload
-        def __new__(cls, __type: type[T], *, config: ConfigDict | None = ...) -> TypeAdapter[T]:
-            ...
+    # This second overload is for unsupported special forms (such as Annotated, Union, etc.)
+    # Currently there is no way to type this correctly
+    # See https://github.com/python/typing/pull/1618
+    @overload
+    def __init__(
+        self: TypeAdapter[Any],
+        type: Any,
+        *,
+        config: ConfigDict | None = ...,
+        _parent_depth: int = ...,
+        module: str | None = ...,
+    ) -> None:
+        ...
 
-        # this overload is for non-type things like Union[int, str]
-        # Pyright currently handles this "correctly", but MyPy understands this as TypeAdapter[object]
-        # so an explicit type cast is needed
-        @overload
-        def __new__(cls, __type: T, *, config: ConfigDict | None = ...) -> TypeAdapter[T]:
-            ...
+    def __init__(
+        self,
+        type: Any,
+        *,
+        config: ConfigDict | None = None,
+        _parent_depth: int = 2,
+        module: str | None = None,
+    ) -> None:
+        """Initializes the TypeAdapter object.
 
-        def __new__(cls, __type: Any, *, config: ConfigDict | None = ...) -> TypeAdapter[T]:
-            """A class representing the type adapter."""
-            raise NotImplementedError
+        Args:
+            type: The type associated with the `TypeAdapter`.
+            config: Configuration for the `TypeAdapter`, should be a dictionary conforming to [`ConfigDict`][pydantic.config.ConfigDict].
+            _parent_depth: depth at which to search the parent namespace to construct the local namespace.
+            module: The module that passes to plugin if provided.
 
-    def __init__(self, type: Any, *, config: ConfigDict | None = None, _parent_depth: int = 2) -> None:
-        """Initializes the TypeAdapter object."""
-        config_wrapper = _config.ConfigWrapper(config)
+        !!! note
+            You cannot use the `config` argument when instantiating a `TypeAdapter` if the type you're using has its own
+            config that cannot be overridden (ex: `BaseModel`, `TypedDict`, and `dataclass`). A
+            [`type-adapter-config-unused`](../errors/usage_errors.md#type-adapter-config-unused) error will be raised in this case.
 
-        try:
-            type_has_config = issubclass(type, BaseModel) or is_dataclass(type) or is_typeddict(type)
-        except TypeError:
-            # type is not a class
-            type_has_config = False
+        !!! note
+            The `_parent_depth` argument is named with an underscore to suggest its private nature and discourage use.
+            It may be deprecated in a minor version, so we only recommend using it if you're
+            comfortable with potential change in behavior / support.
+
+        ??? tip "Compatibility with `mypy`"
+            Depending on the type used, `mypy` might raise an error when instantiating a `TypeAdapter`. As a workaround, you can explicitly
+            annotate your variable:
+
+            ```py
+            from typing import Union
+
+            from pydantic import TypeAdapter
+
+            ta: TypeAdapter[Union[str, int]] = TypeAdapter(Union[str, int])  # type: ignore[arg-type]
+            ```
+
+        Returns:
+            A type adapter configured for the specified `type`.
+        """
+        type_is_annotated: bool = _typing_extra.is_annotated(type)
+        annotated_type: Any = get_args(type)[0] if type_is_annotated else None
+        type_has_config: bool = _type_has_config(annotated_type if type_is_annotated else type)
 
         if type_has_config and config is not None:
             raise PydanticUserError(
@@ -145,27 +203,31 @@ class TypeAdapter(Generic[T]):
                 code='type-adapter-config-unused',
             )
 
+        config_wrapper = _config.ConfigWrapper(config)
+
         core_schema: CoreSchema
         try:
             core_schema = _getattr_no_parents(type, '__pydantic_core_schema__')
         except AttributeError:
             core_schema = _get_schema(type, config_wrapper, parent_depth=_parent_depth + 1)
 
-        core_schema = _discriminated_union.apply_discriminators(_core_utils.flatten_schema_defs(core_schema))
-        simplified_core_schema = _core_utils.inline_schema_defs(core_schema)
-
         core_config = config_wrapper.core_config(None)
         validator: SchemaValidator
         try:
             validator = _getattr_no_parents(type, '__pydantic_validator__')
         except AttributeError:
-            validator = SchemaValidator(simplified_core_schema, core_config)
+            if module is None:
+                f = sys._getframe(1)
+                module = cast(str, f.f_globals.get('__name__', ''))
+            validator = create_schema_validator(
+                core_schema, type, module, str(type), 'TypeAdapter', core_config, config_wrapper.plugin_settings
+            )  # type: ignore
 
         serializer: SchemaSerializer
         try:
             serializer = _getattr_no_parents(type, '__pydantic_serializer__')
         except AttributeError:
-            serializer = SchemaSerializer(simplified_core_schema, core_config)
+            serializer = SchemaSerializer(core_schema, core_config)
 
         self.core_schema = core_schema
         self.validator = validator
@@ -173,7 +235,8 @@ class TypeAdapter(Generic[T]):
 
     def validate_python(
         self,
-        __object: Any,
+        object: Any,
+        /,
         *,
         strict: bool | None = None,
         from_attributes: bool | None = None,
@@ -182,30 +245,49 @@ class TypeAdapter(Generic[T]):
         """Validate a Python object against the model.
 
         Args:
-            __object: The Python object to validate against the model.
+            object: The Python object to validate against the model.
             strict: Whether to strictly check types.
             from_attributes: Whether to extract data from object attributes.
             context: Additional context to pass to the validator.
 
+        !!! note
+            When using `TypeAdapter` with a Pydantic `dataclass`, the use of the `from_attributes`
+            argument is not supported.
+
         Returns:
             The validated object.
         """
-        return self.validator.validate_python(__object, strict=strict, from_attributes=from_attributes, context=context)
+        return self.validator.validate_python(object, strict=strict, from_attributes=from_attributes, context=context)
 
     def validate_json(
-        self, __data: str | bytes, *, strict: bool | None = None, context: dict[str, Any] | None = None
+        self, data: str | bytes, /, *, strict: bool | None = None, context: dict[str, Any] | None = None
     ) -> T:
-        """Validate a JSON string or bytes against the model.
+        """Usage docs: https://docs.pydantic.dev/2.7/concepts/json/#json-parsing
+
+        Validate a JSON string or bytes against the model.
 
         Args:
-            __data: The JSON data to validate against the model.
+            data: The JSON data to validate against the model.
             strict: Whether to strictly check types.
             context: Additional context to use during validation.
 
         Returns:
             The validated object.
         """
-        return self.validator.validate_json(__data, strict=strict, context=context)
+        return self.validator.validate_json(data, strict=strict, context=context)
+
+    def validate_strings(self, obj: Any, /, *, strict: bool | None = None, context: dict[str, Any] | None = None) -> T:
+        """Validate object contains string data against the model.
+
+        Args:
+            obj: The object contains string data to validate.
+            strict: Whether to strictly check types.
+            context: Additional context to use during validation.
+
+        Returns:
+            The validated object.
+        """
+        return self.validator.validate_strings(obj, strict=strict, context=context)
 
     def get_default_value(self, *, strict: bool | None = None, context: dict[str, Any] | None = None) -> Some[T] | None:
         """Get the default value for the wrapped type.
@@ -221,7 +303,8 @@ class TypeAdapter(Generic[T]):
 
     def dump_python(
         self,
-        __instance: T,
+        instance: T,
+        /,
         *,
         mode: Literal['json', 'python'] = 'python',
         include: IncEx | None = None,
@@ -231,12 +314,13 @@ class TypeAdapter(Generic[T]):
         exclude_defaults: bool = False,
         exclude_none: bool = False,
         round_trip: bool = False,
-        warnings: bool = True,
+        warnings: bool | Literal['none', 'warn', 'error'] = True,
+        serialize_as_any: bool = False,
     ) -> Any:
         """Dump an instance of the adapted type to a Python object.
 
         Args:
-            __instance: The Python object to serialize.
+            instance: The Python object to serialize.
             mode: The output format.
             include: Fields to include in the output.
             exclude: Fields to exclude from the output.
@@ -245,13 +329,15 @@ class TypeAdapter(Generic[T]):
             exclude_defaults: Whether to exclude fields with default values.
             exclude_none: Whether to exclude fields with None values.
             round_trip: Whether to output the serialized data in a way that is compatible with deserialization.
-            warnings: Whether to display serialization warnings.
+            warnings: How to handle serialization errors. False/"none" ignores them, True/"warn" logs errors,
+                "error" raises a [`PydanticSerializationError`][pydantic_core.PydanticSerializationError].
+            serialize_as_any: Whether to serialize fields with duck-typing serialization behavior.
 
         Returns:
             The serialized object.
         """
         return self.serializer.to_python(
-            __instance,
+            instance,
             mode=mode,
             by_alias=by_alias,
             include=include,
@@ -261,11 +347,13 @@ class TypeAdapter(Generic[T]):
             exclude_none=exclude_none,
             round_trip=round_trip,
             warnings=warnings,
+            serialize_as_any=serialize_as_any,
         )
 
     def dump_json(
         self,
-        __instance: T,
+        instance: T,
+        /,
         *,
         indent: int | None = None,
         include: IncEx | None = None,
@@ -275,12 +363,15 @@ class TypeAdapter(Generic[T]):
         exclude_defaults: bool = False,
         exclude_none: bool = False,
         round_trip: bool = False,
-        warnings: bool = True,
+        warnings: bool | Literal['none', 'warn', 'error'] = True,
+        serialize_as_any: bool = False,
     ) -> bytes:
-        """Serialize an instance of the adapted type to JSON.
+        """Usage docs: https://docs.pydantic.dev/2.7/concepts/json/#json-serialization
+
+        Serialize an instance of the adapted type to JSON.
 
         Args:
-            __instance: The instance to be serialized.
+            instance: The instance to be serialized.
             indent: Number of spaces for JSON indentation.
             include: Fields to include.
             exclude: Fields to exclude.
@@ -289,13 +380,15 @@ class TypeAdapter(Generic[T]):
             exclude_defaults: Whether to exclude fields with default values.
             exclude_none: Whether to exclude fields with a value of `None`.
             round_trip: Whether to serialize and deserialize the instance to ensure round-tripping.
-            warnings: Whether to emit serialization warnings.
+            warnings: How to handle serialization errors. False/"none" ignores them, True/"warn" logs errors,
+                "error" raises a [`PydanticSerializationError`][pydantic_core.PydanticSerializationError].
+            serialize_as_any: Whether to serialize fields with duck-typing serialization behavior.
 
         Returns:
             The JSON representation of the given instance as bytes.
         """
         return self.serializer.to_json(
-            __instance,
+            instance,
             indent=indent,
             include=include,
             exclude=exclude,
@@ -305,6 +398,7 @@ class TypeAdapter(Generic[T]):
             exclude_none=exclude_none,
             round_trip=round_trip,
             warnings=warnings,
+            serialize_as_any=serialize_as_any,
         )
 
     def json_schema(
@@ -331,18 +425,19 @@ class TypeAdapter(Generic[T]):
 
     @staticmethod
     def json_schemas(
-        __inputs: Iterable[tuple[JsonSchemaKeyT, JsonSchemaMode, TypeAdapter[Any]]],
+        inputs: Iterable[tuple[JsonSchemaKeyT, JsonSchemaMode, TypeAdapter[Any]]],
+        /,
         *,
         by_alias: bool = True,
         title: str | None = None,
         description: str | None = None,
         ref_template: str = DEFAULT_REF_TEMPLATE,
         schema_generator: type[GenerateJsonSchema] = GenerateJsonSchema,
-    ) -> tuple[dict[tuple[JsonSchemaKeyT, JsonSchemaMode], DefsRef], JsonSchemaValue]:
+    ) -> tuple[dict[tuple[JsonSchemaKeyT, JsonSchemaMode], JsonSchemaValue], JsonSchemaValue]:
         """Generate a JSON schema including definitions from multiple type adapters.
 
         Args:
-            __inputs: Inputs to schema generation. The first two items will form the keys of the (first)
+            inputs: Inputs to schema generation. The first two items will form the keys of the (first)
                 output mapping; the type adapters will provide the core schemas that get converted into
                 definitions in the output JSON schema.
             by_alias: Whether to use alias names.
@@ -352,14 +447,20 @@ class TypeAdapter(Generic[T]):
             schema_generator: The generator class used for creating the schema.
 
         Returns:
-            The first item contains the mapping of key + mode to a definitions reference, which will be a key
-                of the $defs mapping in the JSON schema included as the second member of the returned tuple.
+            A tuple where:
+
+                - The first element is a dictionary whose keys are tuples of JSON schema key type and JSON mode, and
+                    whose values are the JSON schema corresponding to that pair of inputs. (These schemas may have
+                    JsonRef references to definitions that are defined in the second returned element.)
+                - The second element is a JSON schema containing all definitions referenced in the first returned
+                    element, along with the optional title and description keys.
+
         """
         schema_generator_instance = schema_generator(by_alias=by_alias, ref_template=ref_template)
 
-        inputs = [(key, mode, adapter.core_schema) for key, mode, adapter in __inputs]
+        inputs_ = [(key, mode, adapter.core_schema) for key, mode, adapter in inputs]
 
-        key_map, definitions = schema_generator_instance.generate_definitions(inputs)
+        json_schemas_map, definitions = schema_generator_instance.generate_definitions(inputs_)
 
         json_schema: dict[str, Any] = {}
         if definitions:
@@ -369,4 +470,4 @@ class TypeAdapter(Generic[T]):
         if description:
             json_schema['description'] = description
 
-        return key_map, json_schema
+        return json_schemas_map, json_schema
