@@ -4,14 +4,14 @@ Test mypy failure with missing attribute
 from datetime import datetime
 from typing import List, Optional
 
-from pydantic import BaseModel, NoneStr
+from pydantic import BaseModel
 from pydantic.types import Json
 
 
 class Model(BaseModel):
     age: int
     first_name = 'John'
-    last_name: NoneStr = None
+    last_name: Optional[str] = None
     signup_ts: Optional[datetime] = None
     list_of_ints: List[int]
     json_list_of_ints: Json[List[int]]
