@@ -4,13 +4,13 @@ Test mypy failure with invalid types.
 from datetime import datetime
 from typing import List, Optional
 
-from pydantic import BaseModel, NoneStr
+from pydantic import BaseModel
 
 
 class Model(BaseModel):
     age: int
     first_name = 'John'
-    last_name: NoneStr = None
+    last_name: Optional[str] = None
     signup_ts: Optional[datetime] = None
     list_of_ints: List[int]
 
