@@ -1,71 +1,82 @@
-from typing import Any, Generic, List, Optional, Set, TypeVar, Union
+from typing import Generic, List, Optional, Set, TypeVar, Union
 
-from pydantic import BaseModel, BaseSettings, Extra, Field, validator
+from pydantic import BaseModel, ConfigDict, Extra, Field, field_validator
 from pydantic.dataclasses import dataclass
-from pydantic.generics import GenericModel
 
 
 class Model(BaseModel):
+    model_config = ConfigDict(alias_generator=None, frozen=True, extra=Extra.forbid)
     x: int
     y: str
 
     def method(self) -> None:
         pass
 
-    class Config:
-        alias_generator = None
-        allow_mutation = False
-        extra = Extra.forbid
-
-        def config_method(self) -> None:
-            ...
-
 
 model = Model(x=1, y='y', z='z')
 model = Model(x=1)
 model.y = 'a'
 Model.from_orm({})
-Model.from_orm({})  # type: ignore[pydantic-orm]  # noqa F821
+
+
+class KwargsModel(BaseModel, alias_generator=None, frozen=True, extra=Extra.forbid):
+    x: int
+    y: str
+
+    def method(self) -> None:
+        pass
+
+
+kwargs_model = KwargsModel(x=1, y='y', z='z')
+kwargs_model = KwargsModel(x=1)
+kwargs_model.y = 'a'
+KwargsModel.from_orm({})
 
 
 class ForbidExtraModel(BaseModel):
-    class Config:
-        extra = 'forbid'
+    model_config = ConfigDict(extra=Extra.forbid)
 
 
 ForbidExtraModel(x=1)
 
 
-class ForbidExtraModel2(BaseModel):
-    class Config:
-        extra = 'forbid'
-        validate_all = False
-
-    Config.validate_all = True
+class KwargsForbidExtraModel(BaseModel, extra='forbid'):
+    pass
 
 
-ForbidExtraModel2(x=1)
+KwargsForbidExtraModel(x=1)
 
 
 class BadExtraModel(BaseModel):
-    class Config:
-        extra = 1  # type: ignore[pydantic-config]  # noqa F821
-        extra = 1
+    model_config = ConfigDict(extra=1)  # type: ignore[typeddict-item]
+
+
+class KwargsBadExtraModel(BaseModel, extra=1):
+    pass
 
 
 class BadConfig1(BaseModel):
-    class Config:
-        orm_mode: Any = {}  # not sensible, but should still be handled gracefully
+    model_config = ConfigDict(from_attributes={})  # type: ignore[typeddict-item]
+
+
+class KwargsBadConfig1(BaseModel, from_attributes={}):
+    pass
 
 
 class BadConfig2(BaseModel):
-    class Config:
-        orm_mode = list  # not sensible, but should still be handled gracefully
+    model_config = ConfigDict(from_attributes=list)  # type: ignore[typeddict-item]
+
+
+class KwargsBadConfig2(BaseModel, from_attributes=list):
+    pass
 
 
 class InheritingModel(Model):
-    class Config:
-        allow_mutation = True
+    model_config = ConfigDict(frozen=False)
+
+
+class KwargsInheritingModel(KwargsModel, frozen=False):
+    pass
 
 
 class DefaultTestingModel(BaseModel):
@@ -94,19 +105,12 @@ class UndefinedAnnotationModel(BaseModel):
 UndefinedAnnotationModel()
 
 
-class Settings(BaseSettings):
-    x: int
-
-
-Model.construct(x=1)
-Model.construct(_fields_set={'x'}, x=1, y='2')
-Model.construct(x='1', y='2')
-
-Settings()  # should pass here due to possibly reading from environment
+Model.model_construct(x=1)
+Model.model_construct(_fields_set={'x'}, x=1, y='2')
+Model.model_construct(x='1', y='2')
 
 # Strict mode fails
 inheriting = InheritingModel(x='1', y='1')
-Settings(x='1')
 Model(x='1', y='2')
 
 
@@ -118,7 +122,7 @@ class Blah(BaseModel):
 T = TypeVar('T')
 
 
-class Response(GenericModel, Generic[T]):
+class Response(BaseModel, Generic[T]):
     data: T
     error: Optional[str]
 
@@ -149,19 +153,26 @@ class DynamicAliasModel2(BaseModel):
     x: str = Field(..., alias=x_alias)
     z: int
 
-    class Config:
-        allow_population_by_field_name = True
+    model_config = ConfigDict(populate_by_name=True)
 
 
 DynamicAliasModel2(y='y', z=1)
 DynamicAliasModel2(x='y', z=1)
 
 
+class KwargsDynamicAliasModel(BaseModel, populate_by_name=True):
+    x: str = Field(..., alias=x_alias)
+    z: int
+
+
+KwargsDynamicAliasModel(y='y', z=1)
+KwargsDynamicAliasModel(x='y', z=1)
+
+
 class AliasGeneratorModel(BaseModel):
     x: int
 
-    class Config:
-        alias_generator = lambda x: x + '_'  # noqa E731
+    model_config = ConfigDict(alias_generator=lambda x: x + '_')
 
 
 AliasGeneratorModel(x=1)
@@ -172,20 +183,36 @@ AliasGeneratorModel(z=1)
 class AliasGeneratorModel2(BaseModel):
     x: int = Field(..., alias='y')
 
-    class Config:  # type: ignore[pydantic-alias]  # noqa F821
-        alias_generator = lambda x: x + '_'  # noqa E731
+    model_config = ConfigDict(alias_generator=lambda x: x + '_')  # type: ignore[pydantic-alias]
 
 
 class UntypedFieldModel(BaseModel):
     x: int = 1
     y = 2
-    z = 2  # type: ignore[pydantic-field]  # noqa F821
+    z = 2  # type: ignore[pydantic-field]
 
 
 AliasGeneratorModel2(x=1)
 AliasGeneratorModel2(y=1, z=1)
 
 
+class KwargsAliasGeneratorModel(BaseModel, alias_generator=lambda x: x + '_'):
+    x: int
+
+
+KwargsAliasGeneratorModel(x=1)
+KwargsAliasGeneratorModel(x_=1)
+KwargsAliasGeneratorModel(z=1)
+
+
+class KwargsAliasGeneratorModel2(BaseModel, alias_generator=lambda x: x + '_'):
+    x: int = Field(..., alias='y')
+
+
+KwargsAliasGeneratorModel2(x=1)
+KwargsAliasGeneratorModel2(y=1, z=1)
+
+
 class CoverageTester(Missing):  # noqa F821
     def from_orm(self) -> None:
         pass
@@ -209,10 +236,7 @@ class FrozenModel(BaseModel):
     x: int
     y: str
 
-    class Config:
-        alias_generator = None
-        frozen = True
-        extra = Extra.forbid
+    model_config = ConfigDict(alias_generator=None, frozen=True, extra=Extra.forbid)
 
 
 frozenmodel = FrozenModel(x=1, y='b')
@@ -220,8 +244,7 @@ frozenmodel.y = 'a'
 
 
 class InheritingModel2(FrozenModel):
-    class Config:
-        frozen = False
+    model_config = ConfigDict(frozen=False)
 
 
 inheriting2 = InheritingModel2(x=1, y='c')
@@ -229,7 +252,7 @@ inheriting2.y = 'd'
 
 
 def _default_factory() -> str:
-    ...
+    return 'x'
 
 
 test: List[str] = []
@@ -244,7 +267,7 @@ class FieldDefaultTestingModel(BaseModel):
     g: str = Field(default_factory=set)
     h: int = Field(default_factory=_default_factory)
     i: List[int] = Field(default_factory=list)
-    l: str = Field(default_factory=3)
+    l_: str = Field(default_factory=3)
 
     # Default and default factory
     m: int = Field(default=1, default_factory=list)
@@ -253,7 +276,7 @@ class FieldDefaultTestingModel(BaseModel):
 class ModelWithAnnotatedValidator(BaseModel):
     name: str
 
-    @validator('name')
+    @field_validator('name')
     def noop_validator_with_annotations(self, name: str) -> str:
         # This is a mistake: the first argument to a validator is the class itself,
         # like a classmethod.
