@@ -488,7 +488,7 @@ def test_modify_schema_error():
     with pytest.raises(
         PydanticUserError,
         match='The `__modify_schema__` method is not supported in Pydantic v2. '
-        'Use `__get_pydantic_json_schema__` instead.',
+        'Use `__get_pydantic_json_schema__` instead in class `Model`.',
     ):
 
         class Model(BaseModel):
@@ -497,6 +497,24 @@ def test_modify_schema_error():
                 pass
 
 
+def test_modify_schema_on_nested_class_error() -> None:
+    class SomeLongName:
+        @classmethod
+        def __modify_schema__(cls, field_schema):
+            pass
+
+    with pytest.raises(
+        PydanticUserError,
+        match='The `__modify_schema__` method is not supported in Pydantic v2. '
+        'Use `__get_pydantic_json_schema__` instead in class `SomeLongName`.',
+    ):
+
+        class B(BaseModel):
+            model_config = ConfigDict(arbitrary_types_allowed=True)
+
+            a: SomeLongName
+
+
 def test_v1_v2_custom_type_compatibility() -> None:
     """Create a custom type that works with V1 and V2"""
 
