@@ -18,4 +18,5 @@ cd .. && pip install . && cd fastapi
 # To skip a specific test, add '--deselect path/to/test.py::test_name' to the end of this command
 #
 # To update the list of deselected tests, remove all deselections, run the tests, and re-add any remaining failures
-./scripts/test.sh -vv
+# TODO remove this once that test is fixed, see https://github.com/pydantic/pydantic/pull/9064
+./scripts/test.sh -vv --deselect tests/test_tutorial/test_path_params/test_tutorial005.py::test_get_enums_invalid
