@@ -1,3 +1,4 @@
+import json
 from typing import Union
 
 import pytest
@@ -8,8 +9,10 @@ from pydantic import (
     AmqpDsn,
     AnyUrl,
     BaseModel,
+    ClickHouseDsn,
     CockroachDsn,
     FileUrl,
+    FtpUrl,
     HttpUrl,
     KafkaDsn,
     MariaDBDsn,
@@ -22,6 +25,7 @@ from pydantic import (
     Strict,
     UrlConstraints,
     ValidationError,
+    WebsocketUrl,
 )
 from pydantic.networks import validate_email
 
@@ -366,6 +370,104 @@ def test_http_urls_default_port(url, expected_port, expected_str):
     assert str(m.v) == expected_str
 
 
+@pytest.mark.parametrize(
+    'value,expected',
+    [
+        ('ws://example.com', 'ws://example.com/'),
+        ('wss://example.com', 'wss://example.com/'),
+        ('wss://ws.example.com/', 'wss://ws.example.com/'),
+        ('ws://ws.example.com/', 'ws://ws.example.com/'),
+        ('ws://example.com:8080', 'ws://example.com:8080/'),
+        ('ws://example.com/path', 'ws://example.com/path'),
+        ('wss://example.com:4433', 'wss://example.com:4433/'),
+        ('wss://example.com/path', 'wss://example.com/path'),
+    ],
+)
+def test_websocket_url_success(value, expected):
+    class Schema(BaseModel):
+        ws: WebsocketUrl
+
+    assert Schema(ws=value).ws.unicode_string() == expected
+
+
+@pytest.mark.parametrize(
+    'value,expected',
+    [
+        ('ws://example.com', 80),
+        ('wss://example.com', 443),
+        ('wss://ws.example.com/', 443),
+        ('ws://ws.example.com/', 80),
+        ('ws://example.com:8080', 8080),
+        ('ws://example.com:9999/path', 9999),
+        ('wss://example.com:4433', 4433),
+        ('wss://example.com/path', 443),
+    ],
+)
+def test_websocket_url_port_success(value, expected):
+    class Schema(BaseModel):
+        ws: WebsocketUrl
+
+    assert Schema(ws=value).ws.port == expected
+
+
+@pytest.mark.parametrize(
+    'value,expected',
+    [
+        ('ws://example.com', '/'),
+        ('wss://example.com', '/'),
+        ('wss://ws.example.com/', '/'),
+        ('ws://ws.example.com/', '/'),
+        ('ws://example.com:8080', '/'),
+        ('ws://example.com:9999/path', '/path'),
+        ('wss://example.com:4433', '/'),
+        ('wss://example.com/path/to/ws', '/path/to/ws'),
+    ],
+)
+def test_websocket_url_path_success(value, expected):
+    class Schema(BaseModel):
+        ws: WebsocketUrl
+
+    assert Schema(ws=value).ws.path == expected
+
+
+@pytest.mark.parametrize(
+    'value,expected',
+    [
+        ('ftp://example.com', 'ftp://example.com/'),
+        ('ftp://example.com/path/to/ftp', 'ftp://example.com/path/to/ftp'),
+        ('ftp://example.com:21', 'ftp://example.com/'),
+        ('ftp://example.com:21/path/to/ftp', 'ftp://example.com/path/to/ftp'),
+        ('ftp://example.com', 'ftp://example.com/'),
+        ('ftp://example.com/path/to/ftp', 'ftp://example.com/path/to/ftp'),
+        ('ftp://example.com:990', 'ftp://example.com:990/'),
+        ('ftp://example.com:990/path/to/ftp', 'ftp://example.com:990/path/to/ftp'),
+    ],
+)
+def test_ftp_url_success(value, expected):
+    class Schema(BaseModel):
+        ftp: FtpUrl
+
+    assert Schema(ftp=value).ftp.unicode_string() == expected
+
+
+@pytest.mark.parametrize(
+    'value,expected',
+    [
+        ('ftp://example.com', 21),
+        ('ftp://example.com/path/to/ftp', 21),
+        ('ftp://example.com:21', 21),
+        ('ftp://exaMplФ.com:221/path/to/ftp', 221),
+        ('ftp://example.com:144', 144),
+        ('ftp://example.com:990/path/to/ftp', 990),
+    ],
+)
+def test_ftp_url_port_success(value, expected):
+    class Schema(BaseModel):
+        ftp: FtpUrl
+
+    assert Schema(ftp=value).ftp.port == expected
+
+
 @pytest.mark.parametrize(
     'dsn',
     [
@@ -418,6 +520,20 @@ def test_mariadb_dsns(dsn):
     assert str(Model(a=dsn).a) == dsn
 
 
+@pytest.mark.parametrize(
+    'dsn',
+    [
+        'clickhouse+native://user:pass@localhost:9000/app',
+        'clickhouse+asynch://user:pass@localhost:9000/app',
+    ],
+)
+def test_clickhouse_dsns(dsn):
+    class Model(BaseModel):
+        a: ClickHouseDsn
+
+    assert str(Model(a=dsn).a) == dsn
+
+
 @pytest.mark.parametrize(
     'dsn,error_message',
     (
@@ -850,11 +966,33 @@ def test_name_email():
 
     assert str(Model(v=NameEmail('foo bar', 'foobaR@example.com')).v) == 'foo bar <foobaR@example.com>'
     assert str(Model(v='foo bar  <foobaR@example.com>').v) == 'foo bar <foobaR@example.com>'
+    assert str(Model(v='foobaR@example.com').v) == 'foobaR <foobaR@example.com>'
     assert NameEmail('foo bar', 'foobaR@example.com') == NameEmail('foo bar', 'foobaR@example.com')
     assert NameEmail('foo bar', 'foobaR@example.com') != NameEmail('foo bar', 'different@example.com')
 
+    assert Model.model_validate_json('{"v":"foo bar <foobaR@example.com>"}').v == NameEmail(
+        'foo bar', 'foobaR@example.com'
+    )
+    assert str(Model.model_validate_json('{"v":"foobaR@example.com"}').v) == 'foobaR <foobaR@example.com>'
+    assert (
+        Model(v=NameEmail('foo bar', 'foobaR@example.com')).model_dump_json() == '{"v":"foo bar <foobaR@example.com>"}'
+    )
+
     with pytest.raises(ValidationError) as exc_info:
         Model(v=1)
     assert exc_info.value.errors() == [
         {'input': 1, 'loc': ('v',), 'msg': 'Input is not a valid NameEmail', 'type': 'name_email_type'}
     ]
+
+
+@pytest.mark.skipif(not email_validator, reason='email_validator not installed')
+def test_name_email_serialization():
+    class Model(BaseModel):
+        email: NameEmail
+
+    m = Model.model_validate({'email': '"name@mailbox.com" <name@mailbox.com>'})
+    assert m.email.name == 'name@mailbox.com'
+    assert str(m.email) == '"name@mailbox.com" <name@mailbox.com>'
+
+    obj = json.loads(m.model_dump_json())
+    Model(email=obj['email'])
