@@ -1,14 +1,11 @@
+import json
 from ipaddress import IPv4Address, IPv4Interface, IPv4Network, IPv6Address, IPv6Interface, IPv6Network
+from typing import Any, List
 
 import pytest
 
 from pydantic import BaseModel, IPvAnyAddress, IPvAnyInterface, IPvAnyNetwork, ValidationError
-
-#
-# ipaddress.IPv4Address
-# ipaddress.IPv6Address
-# pydantic.IPvAnyAddress
-#
+from pydantic.config import ConfigDict
 
 
 @pytest.mark.parametrize(
@@ -78,6 +75,102 @@ def test_ipv4address_success(value):
     assert Model(ipv4=value).ipv4 == IPv4Address(value)
 
 
+@pytest.mark.parametrize(
+    'tp,value,errors',
+    [
+        (
+            IPv4Address,
+            IPv4Address('0.0.0.0'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv4Address',
+                    'input': '0.0.0.0',
+                    'ctx': {'class': 'IPv4Address'},
+                }
+            ],
+        ),
+        (
+            IPv4Interface,
+            IPv4Interface('192.168.0.0/24'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv4Interface',
+                    'input': '192.168.0.0/24',
+                    'ctx': {'class': 'IPv4Interface'},
+                }
+            ],
+        ),
+        (
+            IPv4Network,
+            IPv4Network('192.168.0.0/24'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv4Network',
+                    'input': '192.168.0.0/24',
+                    'ctx': {'class': 'IPv4Network'},
+                }
+            ],
+        ),
+        (
+            IPv6Address,
+            IPv6Address('::1:0:1'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv6Address',
+                    'input': '::1:0:1',
+                    'ctx': {'class': 'IPv6Address'},
+                }
+            ],
+        ),
+        (
+            IPv6Interface,
+            IPv6Interface('2001:db00::0/120'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv6Interface',
+                    'input': '2001:db00::/120',
+                    'ctx': {'class': 'IPv6Interface'},
+                }
+            ],
+        ),
+        (
+            IPv6Network,
+            IPv6Network('2001:db00::0/120'),
+            [
+                {
+                    'type': 'is_instance_of',
+                    'loc': ('v',),
+                    'msg': 'Input should be an instance of IPv6Network',
+                    'input': '2001:db00::/120',
+                    'ctx': {'class': 'IPv6Network'},
+                }
+            ],
+        ),
+    ],
+)
+def test_ip_strict(tp: Any, value: Any, errors: List[Any]) -> None:
+    class Model(BaseModel):
+        v: tp
+
+        model_config = ConfigDict(strict=True)
+
+    with pytest.raises(ValidationError) as exc_info:
+        Model(v=str(value))
+    assert exc_info.value.errors(include_url=False) == errors
+
+    assert Model(v=value).v == value
+
+
 @pytest.mark.parametrize(
     'value',
     [
@@ -98,103 +191,53 @@ def test_ipv6address_success(value):
     assert Model(ipv6=value).ipv6 == IPv6Address(value)
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 address', 'type': 'value_error.ipvanyaddress'}],
-        ),
-        (
-            '192.168.0.1.1.1',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 address', 'type': 'value_error.ipvanyaddress'}],
-        ),
-        (
-            -1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 address', 'type': 'value_error.ipvanyaddress'}],
-        ),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 address', 'type': 'value_error.ipvanyaddress'}],
-        ),
-    ],
-)
-def test_ipaddress_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1', -1, 2**128 + 1])
+def test_ipaddress_fails(value):
     class Model(BaseModel):
         ip: IPvAnyAddress
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+    assert exc_info.value.error_count() == 1
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_any_address',
+        'loc': ('ip',),
+        'msg': 'value is not a valid IPv4 or IPv6 address',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ipv4',), 'msg': 'value is not a valid IPv4 address', 'type': 'value_error.ipv4address'}],
-        ),
-        (
-            '192.168.0.1.1.1',
-            [{'loc': ('ipv4',), 'msg': 'value is not a valid IPv4 address', 'type': 'value_error.ipv4address'}],
-        ),
-        (-1, [{'loc': ('ipv4',), 'msg': 'value is not a valid IPv4 address', 'type': 'value_error.ipv4address'}]),
-        (
-            2**32 + 1,
-            [{'loc': ('ipv4',), 'msg': 'value is not a valid IPv4 address', 'type': 'value_error.ipv4address'}],
-        ),
-        (
-            IPv6Address('::0:1:0'),
-            [{'loc': ('ipv4',), 'msg': 'value is not a valid IPv4 address', 'type': 'value_error.ipv4address'}],
-        ),
-    ],
-)
-def test_ipv4address_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1', -1, 2**32 + 1, IPv6Address('::0:1:0')])
+def test_ipv4address_fails(value):
     class Model(BaseModel):
         ipv4: IPv4Address
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ipv4=value)
-    assert exc_info.value.errors() == errors
+    assert exc_info.value.error_count() == 1
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v4_address',
+        'loc': ('ipv4',),
+        'msg': 'Input is not a valid IPv4 address',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ipv6',), 'msg': 'value is not a valid IPv6 address', 'type': 'value_error.ipv6address'}],
-        ),
-        (
-            '192.168.0.1.1.1',
-            [{'loc': ('ipv6',), 'msg': 'value is not a valid IPv6 address', 'type': 'value_error.ipv6address'}],
-        ),
-        (-1, [{'loc': ('ipv6',), 'msg': 'value is not a valid IPv6 address', 'type': 'value_error.ipv6address'}]),
-        (
-            2**128 + 1,
-            [{'loc': ('ipv6',), 'msg': 'value is not a valid IPv6 address', 'type': 'value_error.ipv6address'}],
-        ),
-        (
-            IPv4Address('192.168.0.1'),
-            [{'loc': ('ipv6',), 'msg': 'value is not a valid IPv6 address', 'type': 'value_error.ipv6address'}],
-        ),
-    ],
-)
-def test_ipv6address_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1', -1, 2**128 + 1, IPv4Address('192.168.0.1')])
+def test_ipv6address_fails(value):
     class Model(BaseModel):
         ipv6: IPv6Address
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ipv6=value)
-    assert exc_info.value.errors() == errors
-
-
-#
-# ipaddress.IPv4Network
-# ipaddress.IPv6Network
-# pydantic.IPvAnyNetwork
-#
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v6_address',
+        'loc': ('ipv6',),
+        'msg': 'Input is not a valid IPv6 address',
+        'input': value,
+    }
 
 
 @pytest.mark.parametrize(
@@ -254,103 +297,70 @@ def test_ip_v6_network_success(value, cls):
     assert Model(ip=value).ip == cls(value)
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 network', 'type': 'value_error.ipvanynetwork'}],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 network', 'type': 'value_error.ipvanynetwork'}],
-        ),
-        (
-            -1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 network', 'type': 'value_error.ipvanynetwork'}],
-        ),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 or IPv6 network', 'type': 'value_error.ipvanynetwork'}],
-        ),
-    ],
-)
-def test_ipnetwork_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1])
+def test_ipnetwork_fails(value):
     class Model(BaseModel):
         ip: IPvAnyNetwork = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_any_network',
+        'loc': ('ip',),
+        'msg': 'value is not a valid IPv4 or IPv6 network',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 network', 'type': 'value_error.ipv4network'}],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 network', 'type': 'value_error.ipv4network'}],
-        ),
-        (-1, [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 network', 'type': 'value_error.ipv4network'}]),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 network', 'type': 'value_error.ipv4network'}],
-        ),
-        (
-            '2001:db00::1/120',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 network', 'type': 'value_error.ipv4network'}],
-        ),
-    ],
-)
-def test_ip_v4_network_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1, '2001:db00::1/120'])
+def test_ip_v4_network_fails(value):
     class Model(BaseModel):
         ip: IPv4Network = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v4_network',
+        'loc': ('ip',),
+        'msg': 'Input is not a valid IPv4 network',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 network', 'type': 'value_error.ipv6network'}],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 network', 'type': 'value_error.ipv6network'}],
-        ),
-        (-1, [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 network', 'type': 'value_error.ipv6network'}]),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 network', 'type': 'value_error.ipv6network'}],
-        ),
-        (
-            '192.168.0.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 network', 'type': 'value_error.ipv6network'}],
-        ),
-    ],
-)
-def test_ip_v6_network_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1, '192.168.0.1/24'])
+def test_ip_v6_network_fails(value):
     class Model(BaseModel):
         ip: IPv6Network = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v6_network',
+        'loc': ('ip',),
+        'msg': 'Input is not a valid IPv6 network',
+        'input': value,
+    }
 
 
-#
-# ipaddress.IPv4Interface
-# ipaddress.IPv6Interface
-# pydantic.IPvAnyInterface
-#
+def test_ipvany_serialization():
+    class Model(BaseModel):
+        address: IPvAnyAddress
+        network: IPvAnyNetwork
+        interface: IPvAnyInterface
+
+    m = Model(address='127.0.0.1', network='192.0.2.0/27', interface='127.0.0.1/32')
+    assert json.loads(m.model_dump_json()) == {
+        'address': '127.0.0.1',
+        'interface': '127.0.0.1/32',
+        'network': '192.0.2.0/27',
+    }
 
 
 @pytest.mark.parametrize(
@@ -363,7 +373,7 @@ def test_ip_v6_network_fails(value, errors):
         ('2001:db00::0/120', IPv6Interface),
         ('2001:db00::1/120', IPv6Interface),
         (2**32 - 1, IPv4Interface),  # no mask equals to mask /32
-        (2**32 - 1, IPv4Interface),  # so ``strict`` has no effect
+        (2**32 - 1, IPv4Interface),  # so `strict` has no effect
         (20_282_409_603_651_670_423_947_251_286_015, IPv6Interface),  # /128
         (20_282_409_603_651_670_423_947_251_286_014, IPv6Interface),
         (b'\xff\xff\xff\xff', IPv4Interface),  # /32
@@ -395,7 +405,7 @@ def test_ipinterface_success(value, cls):
         ('192.168.128.0/30', IPv4Interface),
         ('192.168.128.1/30', IPv4Interface),
         (2**32 - 1, IPv4Interface),  # no mask equals to mask /32
-        (2**32 - 1, IPv4Interface),  # so ``strict`` has no effect
+        (2**32 - 1, IPv4Interface),  # so `strict` has no effect
         (b'\xff\xff\xff\xff', IPv4Interface),  # /32
         (b'\xff\xff\xff\xff', IPv4Interface),
         (('192.168.0.0', 24), IPv4Interface),
@@ -433,109 +443,55 @@ def test_ip_v6_interface_success(value, cls):
     assert Model(ip=value).ip == cls(value)
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [
-                {
-                    'loc': ('ip',),
-                    'msg': 'value is not a valid IPv4 or IPv6 interface',
-                    'type': 'value_error.ipvanyinterface',
-                }
-            ],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [
-                {
-                    'loc': ('ip',),
-                    'msg': 'value is not a valid IPv4 or IPv6 interface',
-                    'type': 'value_error.ipvanyinterface',
-                }
-            ],
-        ),
-        (
-            -1,
-            [
-                {
-                    'loc': ('ip',),
-                    'msg': 'value is not a valid IPv4 or IPv6 interface',
-                    'type': 'value_error.ipvanyinterface',
-                }
-            ],
-        ),
-        (
-            2**128 + 1,
-            [
-                {
-                    'loc': ('ip',),
-                    'msg': 'value is not a valid IPv4 or IPv6 interface',
-                    'type': 'value_error.ipvanyinterface',
-                }
-            ],
-        ),
-    ],
-)
-def test_ipinterface_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1])
+def test_ipinterface_fails(value):
     class Model(BaseModel):
         ip: IPvAnyInterface = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_any_interface',
+        'loc': ('ip',),
+        'msg': 'value is not a valid IPv4 or IPv6 interface',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 interface', 'type': 'value_error.ipv4interface'}],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 interface', 'type': 'value_error.ipv4interface'}],
-        ),
-        (-1, [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 interface', 'type': 'value_error.ipv4interface'}]),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv4 interface', 'type': 'value_error.ipv4interface'}],
-        ),
-    ],
-)
-def test_ip_v4_interface_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1])
+def test_ip_v4_interface_fails(value):
     class Model(BaseModel):
         ip: IPv4Interface = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v4_interface',
+        'loc': ('ip',),
+        'msg': 'Input is not a valid IPv4 interface',
+        'input': value,
+    }
 
 
-@pytest.mark.parametrize(
-    'value,errors',
-    [
-        (
-            'hello,world',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 interface', 'type': 'value_error.ipv6interface'}],
-        ),
-        (
-            '192.168.0.1.1.1/24',
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 interface', 'type': 'value_error.ipv6interface'}],
-        ),
-        (-1, [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 interface', 'type': 'value_error.ipv6interface'}]),
-        (
-            2**128 + 1,
-            [{'loc': ('ip',), 'msg': 'value is not a valid IPv6 interface', 'type': 'value_error.ipv6interface'}],
-        ),
-    ],
-)
-def test_ip_v6_interface_fails(value, errors):
+@pytest.mark.parametrize('value', ['hello,world', '192.168.0.1.1.1/24', -1, 2**128 + 1])
+def test_ip_v6_interface_fails(value):
     class Model(BaseModel):
         ip: IPv6Interface = None
 
     with pytest.raises(ValidationError) as exc_info:
         Model(ip=value)
-    assert exc_info.value.errors() == errors
+
+    assert exc_info.value.error_count() == 1
+    # insert_assert(exc_info.value.errors(include_url=False)[0])
+    assert exc_info.value.errors(include_url=False)[0] == {
+        'type': 'ip_v6_interface',
+        'loc': ('ip',),
+        'msg': 'Input is not a valid IPv6 interface',
+        'input': value,
+    }
