@@ -4,8 +4,9 @@ New tests for v2 of serialization logic.
 import json
 import re
 import sys
+from enum import Enum
 from functools import partial, partialmethod
-from typing import Any, Callable, ClassVar, Dict, Optional, Pattern
+from typing import Any, Callable, ClassVar, Dict, List, Optional, Pattern, Union
 
 import pytest
 from pydantic_core import PydanticSerializationError, core_schema, to_jsonable_python
@@ -16,9 +17,7 @@ from pydantic import (
     Field,
     FieldSerializationInfo,
     PydanticUserError,
-    SecretStr,
     SerializationInfo,
-    SerializeAsAny,
     SerializerFunctionWrapHandler,
     TypeAdapter,
     computed_field,
@@ -559,6 +558,18 @@ def test_field_multiple_serializer_subclass():
     assert MySubModel(x=1234).model_dump() == {'x': '1234'}
 
 
+def test_serialize_all_fields():
+    class MyModel(BaseModel):
+        x: int
+
+        @field_serializer('*')
+        @classmethod
+        def serialize_all(cls, v: Any):
+            return v * 2
+
+    assert MyModel(x=10).model_dump() == {'x': 20}
+
+
 def int_ser_func_without_info1(v: int, expected: int) -> str:
     return f'{v:,}'
 
@@ -889,28 +900,6 @@ def test_type_adapter_dump_json():
     assert ta.dump_json(Model({'x': 1, 'y': 2.5})) == b'{"x":2,"y":7.5}'
 
 
-def test_serialize_as_any() -> None:
-    class User(BaseModel):
-        name: str
-
-    class UserLogin(User):
-        password: SecretStr
-
-    class OuterModel(BaseModel):
-        maybe_as_any: Optional[SerializeAsAny[User]] = None
-        as_any: SerializeAsAny[User]
-        without: User
-
-    user = UserLogin(name='pydantic', password='password')
-
-    # insert_assert(json.loads(OuterModel(as_any=user, without=user).model_dump_json()))
-    assert json.loads(OuterModel(maybe_as_any=user, as_any=user, without=user).model_dump_json()) == {
-        'maybe_as_any': {'name': 'pydantic', 'password': '**********'},
-        'as_any': {'name': 'pydantic', 'password': '**********'},
-        'without': {'name': 'pydantic'},
-    }
-
-
 @pytest.mark.parametrize('as_annotation', [True, False])
 @pytest.mark.parametrize('mode', ['plain', 'wrap'])
 def test_forward_ref_for_serializers(as_annotation, mode):
@@ -1101,3 +1090,107 @@ def test_forward_ref_for_classmethod_computed_fields():
     }
 
     assert Model().model_dump() == {'two_y': 8}
+
+
+def test_enum_as_dict_key() -> None:
+    # See https://github.com/pydantic/pydantic/issues/7639
+    class MyEnum(Enum):
+        A = 'a'
+        B = 'b'
+
+    class MyModel(BaseModel):
+        foo: Dict[MyEnum, str]
+        bar: MyEnum
+
+    assert MyModel(foo={MyEnum.A: 'hello'}, bar=MyEnum.B).model_dump_json() == '{"foo":{"a":"hello"},"bar":"b"}'
+
+
+def test_subclass_support_unions() -> None:
+    class Pet(BaseModel):
+        name: str
+
+    class Dog(Pet):
+        breed: str
+
+    class Kid(BaseModel):
+        age: str
+
+    class Home(BaseModel):
+        little_guys: Union[List[Pet], List[Kid]]
+
+    class Shelter(BaseModel):
+        pets: List[Pet]
+
+    h1 = Home(little_guys=[Pet(name='spot'), Pet(name='buddy')])
+    assert h1.model_dump() == {'little_guys': [{'name': 'spot'}, {'name': 'buddy'}]}
+
+    h2 = Home(little_guys=[Dog(name='fluffy', breed='lab'), Dog(name='patches', breed='boxer')])
+    assert h2.model_dump() == {'little_guys': [{'name': 'fluffy'}, {'name': 'patches'}]}
+
+    # confirming same serialization + validation behavior as for a single list (not a union)
+    s = Shelter(pets=[Dog(name='fluffy', breed='lab'), Dog(name='patches', breed='boxer')])
+    assert s.model_dump() == {'pets': [{'name': 'fluffy'}, {'name': 'patches'}]}
+
+
+def test_subclass_support_unions_with_forward_ref() -> None:
+    class Bar(BaseModel):
+        bar_id: int
+
+    class Baz(Bar):
+        baz_id: int
+
+    class Foo(BaseModel):
+        items: Union[List['Foo'], List[Bar]]
+
+    foo = Foo(items=[Baz(bar_id=1, baz_id=2), Baz(bar_id=3, baz_id=4)])
+    assert foo.model_dump() == {'items': [{'bar_id': 1}, {'bar_id': 3}]}
+
+    foo_recursive = Foo(items=[Foo(items=[Baz(bar_id=42, baz_id=99)])])
+    assert foo_recursive.model_dump() == {'items': [{'items': [{'bar_id': 42}]}]}
+
+
+def test_serialize_python_context() -> None:
+    contexts: List[Any] = [None, None, {'foo': 'bar'}]
+
+    class Model(BaseModel):
+        x: int
+
+        @field_serializer('x')
+        def serialize_x(self, v: int, info: SerializationInfo) -> int:
+            assert info.context == contexts.pop(0)
+            return v
+
+    m = Model.model_construct(**{'x': 1})
+    m.model_dump()
+    m.model_dump(context=None)
+    m.model_dump(context={'foo': 'bar'})
+    assert contexts == []
+
+
+def test_serialize_json_context() -> None:
+    contexts: List[Any] = [None, None, {'foo': 'bar'}]
+
+    class Model(BaseModel):
+        x: int
+
+        @field_serializer('x')
+        def serialize_x(self, v: int, info: SerializationInfo) -> int:
+            assert info.context == contexts.pop(0)
+            return v
+
+    m = Model.model_construct(**{'x': 1})
+    m.model_dump_json()
+    m.model_dump_json(context=None)
+    m.model_dump_json(context={'foo': 'bar'})
+    assert contexts == []
+
+
+def test_plain_serializer_with_std_type() -> None:
+    """Ensure that a plain serializer can be used with a standard type constructor, rather than having to use lambda x: std_type(x)."""
+
+    class MyModel(BaseModel):
+        x: Annotated[int, PlainSerializer(float)]
+
+    m = MyModel(x=1)
+    assert m.model_dump() == {'x': 1.0}
+    assert m.model_dump_json() == '{"x":1.0}'
