@@ -496,6 +496,20 @@ def test_config_arbitrary_types_allowed():
     ]
 
 
+def test_config_strict():
+    @validate_call(config=dict(strict=True))
+    def foo(a: int, b: List[str]):
+        return f'{a}, {b[0]}'
+
+    assert foo(1, ['bar', 'foobar']) == '1, bar'
+    with pytest.raises(ValidationError) as exc_info:
+        foo('foo', ('bar', 'foobar'))
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'int_type', 'loc': (0,), 'msg': 'Input should be a valid integer', 'input': 'foo'},
+        {'type': 'list_type', 'loc': (1,), 'msg': 'Input should be a valid list', 'input': ('bar', 'foobar')},
+    ]
+
+
 def test_annotated_use_of_alias():
     @validate_call
     def foo(a: Annotated[int, Field(alias='b')], c: Annotated[int, Field()], d: Annotated[int, Field(alias='')]):
